#include <openssl/rsa.h>
#include <openssl/pem.h>
#include <string.h>
#include <assert.h>

#include "lc6_ssl.h"

libchat_user* ssl_create_keypair(libchat_user *user) {

	BIGNUM *bn;
	RSA *rsa;
	BIO *pub;
	BIO *priv;
	size_t publen;
	size_t privlen;

	if ( user == NULL ) {
		user = malloc(sizeof(&user));
		assert(user);
		memset(user, 0, sizeof(&user));
	}

	bn = BN_new();
	BN_set_word(bn, RSA_F4);

	rsa = RSA_new();
	RSA_generate_key_ex(rsa, 2048, bn, NULL);

	priv = BIO_new(BIO_s_mem());
	pub = BIO_new(BIO_s_mem());

	PEM_write_bio_RSAPrivateKey(priv, rsa, NULL, NULL, 0, NULL, NULL);
	PEM_write_bio_RSAPublicKey(pub, rsa);

	RSA_free(rsa);

	privlen = BIO_pending(priv);
	publen  = BIO_pending(pub);

	user->priv_key = malloc(privlen+1);
	user->pub_key  = malloc(publen+1);


	BIO_read(priv, user->priv_key, privlen);
	BIO_read(pub,  user->pub_key,  publen);

	user->priv_key[privlen] = '\0';
	user->pub_key[publen]   = '\0';

	BIO_free_all(priv);
	BIO_free_all(pub);
	BN_free(bn);
//	RSA_free(rsa);

	return user;
}
